@@ -6,35 +6,12 @@
 #' @rdname fit-methods
 #' 
 #' @param x defined relationship between model predictors and an outcome.  May
-#' be a model.frame (data.frame) containing a formula, data, and optionally case
-#' weights; a formula; or a recipe.
+#' be a ModelFrame containing a formula, data, and optionally case weights; a
+#' formula; or a recipe.
 #' @param ... arguments passed to other methods.
 #' 
 fit <- function(x, ...) {
-  UseMethod("fit", x)
-}
-
-
-#' @rdname fit-methods
-#' 
-fit.data.frame <- function(x, model, ...) {
-  model <- getMLObject(model, "MLModel")
-  
-  y <- response(x)
-  if (!any(sapply(model@types, function(type) is(y, type)))) {
-    stop("invalid response type '", class(y)[1], "' for ", model@name)
-  }
-  
-  requireModelNamespaces(model@packages)
-  fo <- formula(terms(x))
-  fo[[2]] <- formula(x)[[2]]
-  weights <- model.weights(x)
-  if (is.null(weights)) weights <- rep(1, nrow(x))
-  args <- c(list(formula = fo, data = x, weights = weights), model@params)
-  data_params <- list(y = y, nobs = nrow(x))
-  data_params$nvars <-  model@nvars(x[1, , drop = FALSE])
-  do.call(model@fit, args, envir = list2env(c(args, data_params))) %>%
-    asMLModelFit(paste0(model@name, "Fit"), model)
+  UseMethod("fit")
 }
 
 
@@ -52,20 +29,54 @@ fit.data.frame <- function(x, model, ...) {
 #' @examples
 #' ## Survival response example
 #' library(survival)
+#' library(MASS)
 #' 
-#' gbmfit <- fit(Surv(time, status) ~ age + sex + ph.ecog + ph.karno +
-#'                                    pat.karno + meal.cal + wt.loss,
-#'               data = lung, GBMModel)
+#' gbmfit <- fit(Surv(time, status != 2) ~ sex + age + year + thickness + ulcer,
+#'               data = Melanoma, model = GBMModel)
 #' varimp(gbmfit)
 #' 
 fit.formula <- function(x, data, model, ...) {
-  fit(model.frame(x, data, na.action = NULL), model)
+  fit(ModelFrame(x, data, na.action = na.pass), model)
+}
+
+
+#' @rdname fit-methods
+#' 
+fit.ModelFrame <- function(x, model, ...) {
+  .fit(getMLObject(model, "MLModel"), x)
 }
 
 
 #' @rdname fit-methods
 #' 
 fit.recipe <- function(x, model, ...) {
-  x <- prep(x, retain = TRUE)
-  fit(formula(x), juice(x), model)
+  .fit(getMLObject(model, "MLModel"), x)
+}
+
+
+.fit <- function(model, x, ...) {
+  UseMethod(".fit")
+}
+
+
+.fit.MLModel <- function(model, x, ...) {
+  mf <- ModelFrame(x, na.action = na.pass)
+  
+  y <- response(mf)
+  if (!any(sapply(model@types, function(type) is_response(y, type)))) {
+    stop("invalid response type '", class(y)[1], "' for ", model@name)
+  }
+  
+  requireModelNamespaces(model@packages)
+  fo <- formula(terms(mf))
+  weights <- model.weights(mf)
+  if (is.null(weights)) weights <- rep(1, nrow(mf))
+  args <- c(list(formula = fo, data = mf, weights = weights), model@params)
+  data_params <- list(y = y, nobs = nrow(mf))
+  data_params$nvars <-  model@nvars(mf[1, , drop = FALSE])
+  
+  fitbit(model, "x") <- x
+  fitbit(model, "y") <- y
+  do.call(model@fit, args, envir = list2env(c(args, data_params))) %>%
+    asMLModelFit(paste0(model@name, "Fit"), model)
 }
