@@ -1,6 +1,6 @@
 models <- c("CoxModel", "CoxStepAICModel")
 
-for(model in models) {
+for (model in models) {
   context(model)
   
   test_that("model fitting", {
@@ -10,7 +10,7 @@ for(model in models) {
       expect_error(test_model_factor2(model))
       expect_error(test_model_numeric(model))
       expect_error(test_model_ordered(model))
-      test_model_Surv(model)
+      expect_output(test_model_Surv(model))
     })
   })
 }
