@@ -8,7 +8,7 @@ test_that("model fitting", {
     expect_error(test_model_factor(model))
     expect_error(test_model_factor2(model))
     expect_error(test_model_numeric(model))
-    test_model_ordered(model)
+    expect_output(test_model_ordered(model))
     expect_error(test_model_Surv(model))
   })
 })
